import _extends from "@babel/runtime/helpers/extends";
import React, { useEffect, useContext } from "react";
import { PartytownContext } from "./partytown-context";
import { requestIdleCallback } from "./request-idle-callback-shim";
export let ScriptStrategy; // eslint-disable-next-line @typescript-eslint/naming-convention

(function (ScriptStrategy) {
  ScriptStrategy["postHydrate"] = "post-hydrate";
  ScriptStrategy["idle"] = "idle";
  ScriptStrategy["offMainThread"] = "off-main-thread";
})(ScriptStrategy || (ScriptStrategy = {}));

const handledProps = new Set([`src`, `strategy`, `dangerouslySetInnerHTML`, `children`, `onLoad`, `onError`]);
export const scriptCache = new Set();
export const scriptCallbackCache = new Map();
export function Script(props) {
  const {
    id,
    src,
    strategy = ScriptStrategy.postHydrate,
    onLoad,
    onError
  } = props || {};
  const {
    collectScript
  } = useContext(PartytownContext);
  useEffect(() => {
    let script;

    switch (strategy) {
      case ScriptStrategy.postHydrate:
        script = injectScript(props);
        break;

      case ScriptStrategy.idle:
        requestIdleCallback(() => {
          script = injectScript(props);
        });
        break;

      case ScriptStrategy.offMainThread:
        if (collectScript) {
          const attributes = resolveAttributes(props);
          collectScript(attributes);
        }

        break;
    }

    return () => {
      var _script3;

      if (onLoad) {
        var _script;

        (_script = script) === null || _script === void 0 ? void 0 : _script.removeEventListener(`load`, onLoad);
      }

      if (onError) {
        var _script2;

        (_script2 = script) === null || _script2 === void 0 ? void 0 : _script2.removeEventListener(`error`, onError);
      }

      (_script3 = script) === null || _script3 === void 0 ? void 0 : _script3.remove();
    };
  }, []);

  if (strategy === ScriptStrategy.offMainThread) {
    const inlineScript = resolveInlineScript(props);
    const attributes = resolveAttributes(props);

    if (typeof window === `undefined` && collectScript) {
      const identifier = id || src || `no-id-or-src`;
      console.warn(`Unable to collect off-main-thread script '${identifier}' for configuration with Partytown.\nGatsby script components must be used either as a child of your page, in wrapPageElement, or wrapRootElement.\nSee https://gatsby.dev/gatsby-script for more information.`);
      collectScript(attributes);
    }

    if (inlineScript) {
      return /*#__PURE__*/React.createElement("script", _extends({
        type: "text/partytown",
        "data-strategy": strategy,
        crossOrigin: "anonymous"
      }, attributes, {
        dangerouslySetInnerHTML: {
          __html: resolveInlineScript(props)
        }
      }));
    }

    return /*#__PURE__*/React.createElement("script", _extends({
      type: "text/partytown",
      src: proxyPartytownUrl(src),
      "data-strategy": strategy,
      crossOrigin: "anonymous"
    }, attributes));
  }

  return null;
}

function injectScript(props) {
  const {
    id,
    src,
    strategy = ScriptStrategy.postHydrate,
    onLoad,
    onError
  } = props || {};
  const scriptKey = id || src || (scriptCache.size + 1).toString();
  /**
   * If a duplicate script is already loaded/errored, we replay load/error callbacks with the original event.
   * If it's not yet loaded/errored, keep track of callbacks so we can call load/error callbacks for each when the event occurs.
   */

  const cachedCallbacks = scriptCallbackCache.get(scriptKey) || {};
  const callbackNames = [`load`, `error`];
  const currentCallbacks = {
    load: onLoad,
    error: onError
  };

  for (const name of callbackNames) {
    if (currentCallbacks !== null && currentCallbacks !== void 0 && currentCallbacks[name]) {
      var _cachedCallbacks$name;

      const {
        callbacks = []
      } = (cachedCallbacks === null || cachedCallbacks === void 0 ? void 0 : cachedCallbacks[name]) || {};
      callbacks.push(currentCallbacks === null || currentCallbacks === void 0 ? void 0 : currentCallbacks[name]);

      if (cachedCallbacks !== null && cachedCallbacks !== void 0 && (_cachedCallbacks$name = cachedCallbacks[name]) !== null && _cachedCallbacks$name !== void 0 && _cachedCallbacks$name.event) {
        var _currentCallbacks$nam, _cachedCallbacks$name2;

        currentCallbacks === null || currentCallbacks === void 0 ? void 0 : (_currentCallbacks$nam = currentCallbacks[name]) === null || _currentCallbacks$nam === void 0 ? void 0 : _currentCallbacks$nam.call(currentCallbacks, cachedCallbacks === null || cachedCallbacks === void 0 ? void 0 : (_cachedCallbacks$name2 = cachedCallbacks[name]) === null || _cachedCallbacks$name2 === void 0 ? void 0 : _cachedCallbacks$name2.event);
      } else {
        scriptCallbackCache.set(scriptKey, {
          [name]: {
            callbacks
          }
        });
      }
    }
  } // Avoid injecting duplicate scripts into the DOM


  if (scriptCache.has(scriptKey)) {
    return null;
  }

  const inlineScript = resolveInlineScript(props);
  const attributes = resolveAttributes(props);
  const script = document.createElement(`script`);

  if (id) {
    script.id = id;
  }

  script.dataset.strategy = strategy;

  for (const [key, value] of Object.entries(attributes)) {
    script.setAttribute(key, value);
  }

  if (inlineScript) {
    script.textContent = inlineScript;
  }

  if (src) {
    script.src = src;
  }

  for (const name of callbackNames) {
    if (currentCallbacks !== null && currentCallbacks !== void 0 && currentCallbacks[name]) {
      script.addEventListener(name, event => {
        const cachedCallbacks = scriptCallbackCache.get(scriptKey) || {};

        for (const callback of (cachedCallbacks === null || cachedCallbacks === void 0 ? void 0 : (_cachedCallbacks$name3 = cachedCallbacks[name]) === null || _cachedCallbacks$name3 === void 0 ? void 0 : _cachedCallbacks$name3.callbacks) || []) {
          var _cachedCallbacks$name3;

          callback(event);
        }

        scriptCallbackCache.set(scriptKey, {
          [name]: {
            event
          }
        });
      });
    }
  }

  scriptCache.add(scriptKey);
  document.body.appendChild(script);
  scriptCache.add(scriptKey);
  return script;
}

function resolveInlineScript(props) {
  const {
    dangerouslySetInnerHTML,
    children = ``
  } = props || {};
  const {
    __html: dangerousHTML = ``
  } = dangerouslySetInnerHTML || {};
  return dangerousHTML || children;
}

function resolveAttributes(props) {
  const attributes = {};

  for (const [key, value] of Object.entries(props)) {
    if (handledProps.has(key)) {
      continue;
    }

    attributes[key] = value;
  }

  return attributes;
}

function proxyPartytownUrl(url) {
  if (!url) {
    return undefined;
  }

  return `/__third-party-proxy?url=${encodeURIComponent(url)}`;
}